/*
 * Mach Operating System
 * Copyright (c) 1992 Carnegie Mellon University
 * All Rights Reserved.
 *
 * Permission to use, copy, modify and distribute this software and its
 * documentation is hereby granted, provided that both the copyright
 * notice and this permission notice appear in all copies of the
 * software, derivative works or modified versions, and any portions
 * thereof, and that both notices appear in supporting documentation.
 *
 * CARNEGIE MELLON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS"
 * CONDITION.  CARNEGIE MELLON DISCLAIMS ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 *
 * Carnegie Mellon requests users of this software to return to
 *
 *  Software Distribution Coordinator  or  Software.Distribution@CS.CMU.EDU
 *  School of Computer Science
 *  Carnegie Mellon University
 *  Pittsburgh PA 15213-3890
 *
 * any improvements or extensions that they make and grant Carnegie Mellon
 * the rights to redistribute these changes.
 */
/*
 * Generic interrupt priority definitions for m68k processors
 *
 * Author:       Bryan Ford
 * Contributors: Niklas Hallqvist
 */

#ifndef _m68k_machparam_
#define _m68k_machparam_

#include <machine/psl.h>
  
#ifdef SPL_DEBUG

extern int spl7 ();
extern int spl6 ();
extern int spl5 ();
extern int spl4 ();
extern int spl3 ();
extern int spl2 ();
extern int spl1 ();
extern int spl0 ();

extern int spln (unsigned short spl);
extern void splx (unsigned short spl);

#else /* inline definitions then */

#ifdef MASKED_SPL

/* Set the interrupt priority level and save the old level */
static inline int spln (unsigned short spl)
{
  unsigned short sr, scratch;

  asm volatile ("movew sr,%0;"
		"movew %0,%1;"
		"andiw #0xf8ff,%1;"
		"orw %2,%1;"
		"movew %1,sr;"
		: "=d" (sr), "=d" (scratch) : "d" (spl) : "cc");
  return sr & 0x700;
}


/* Set the interrupt priority level back to the previous level */
static inline void splx (unsigned short pri)
{
  unsigned short sr;

  asm volatile ("movew sr,%0;"
		"andiw #0xf8ff,%0;"
		"orw %1,%0;"
		"movew %0,sr;"
		: "=d" (sr) : "d" (pri) : "cc");
}


/* Generic spl definitions for m68k interrupt priorities */
static inline int spl7 (void)
{
  unsigned short sr;

  asm volatile ("movew sr,%0;"
		"oriw #0x0700,sr"
		: "=d" (sr) : : "cc");
  return sr & 0x700;
}

static inline int spl0 (void)
{
  unsigned short sr;

  asm volatile ("movew sr,%0;"
		"andiw #0xf8ff,sr"
		: "=d" (sr) : : "cc");
  return sr & 0x700;
}

#define spl6() spln (0x0600)
#define spl5() spln (0x0500)
#define spl4() spln (0x0400)
#define spl3() spln (0x0300)
#define spl2() spln (0x0200)
#define spl1() spln (0x0100)

#else /* MASKED_SPL */

static inline int spln (unsigned short spl)
{
  unsigned short sr;

  asm volatile ("movew sr,%0;"
		"movew %1,sr;"
		: "=d" (sr) : "d" (spl) : "cc");
  return sr & 0x3700;
}


/* Set the interrupt priority level back to the previous level */
static inline void splx (unsigned short pri)
{
  asm volatile ("movew %0,sr" : : "d" (pri) : "cc");
}

#define spl7() spln (0x2700)
#define spl6() spln (0x2600)
#define spl5() spln (0x2500)
#define spl4() spln (0x2400)
#define spl3() spln (0x2300)
#define spl2() spln (0x2200)
#define spl1() spln (0x2100)
#define spl0() spln (0x2000)

#endif /* MASKED_SPL */

#endif /* SPL_DEBUG */

#endif _m68k_machparam_
